function [alarmResult,varargout]=challenge1(recordName,alarm_type)
%
%  alarmResult=challenge(recordName,alarm_type)
%
% Sample entry for the 2015 PhysioNet/CinC Challenge.
%
% Inputs:
%   recordName
%       String specifying the record name to process
%   alarmType
%       String specifying the alarm type. Alarm types are:
%             Asystole, Bradycardia, Tachycardia,
%             Ventricular_Tachycardia, Ventricular_Flutter_Fib
%
%
% Outputs:
%   alarmResult
%       Integer value where 0 = false alarm and 1 is a true
%       alarm. 
%
%
% To run your entry on the entire training set in a format that is
% compatible with PhysioNet's scoring enviroment, run the script
% generateValidationSet.m
%
% Dependencies:
%
%       1) This function does not requires that you have the WFDB
%       App Toolbox installed. 
%       A matlab function 'rdmat' can read the data instead of using WFDB
%       Toolbox.
%
%       2) The CHALLENGE function requires that you have downloaded the challenge
%       data 'set-p' in a subdirectory of the current directory. The subdirectory
%       should be called '/challenge/2015/set-p/' . The 'set-p' dataset can
%       be downloaded from PhysioNet at:
%           http://physionet.org/physiobank/database/challenge/2015/
%          
%         This dataset is used by the generateValidationSet.m script to
%         create the annotations on your traing set that will be used to 
%         verify that your entry works properly in the PhysioNet testing 
%         environment. 
%
% Version 0.5
%
%
% Written by Qiao Li, November 10, 2014.
% Last Modified: Ikaro Silva February 11, 2015
% 
% More Modified: Kenneth Ball, June 26, 2015
%
%
% %Example using training data- 
% alarmResult=challenge('./challenge/set-p/100','Asystole')
%

% Name of file containing answers
answers = 'answers.txt';

%Get all ECG, blood pressure and photoplethysmogram signals
[~,signal,Fs,siginfo]=rdmat(recordName);
alarmResult=1;
description=squeeze(struct2cell(siginfo));
description=description(4,:);
units=squeeze(struct2cell(siginfo));
units=units(1,:);

abp_ind=get_index(description,'ABP');
ppg_ind=get_index(description,'PLETH');
ecg_ind=get_index(units,'mV');

N_d=Fs*5*60; % alarm position
N0_d=N_d-10*Fs+1; % 10 seconds before alarm
N1 = Fs*2*60; % first 2 minutes. 

if ~isempty(abp_ind)
    try
    test1 = find(isnan(signal(:,ecg_ind)));
    if ~isempty(test1)
        error('Bad ECG Lead');
    end
%     phase_abp = angle(hilbert(signal(:,ecg_ind(1))-mean(signal(:,ecg_ind(1)))));
    phase_abp = angle(hilbert(signal(:,abp_ind)-mean(signal(:,abp_ind))));
    steps = ceil(unwrap(phase_abp,3*pi/2)./(2*pi));
    winBounds = zeros(max(steps),2);
    for ii = 1:max(steps)
        winBounds(ii,1) = find(steps == ii,1,'first');
        winBounds(ii,2) = find(steps == ii,1,'last');
    end
%     preInds = find(winBounds(:,2)<=N0_d);
    preInds = find(winBounds(:,2)<=N1);
    postInds = find(winBounds(:,1)>=N0_d & winBounds(:,1)<N_d);
    hbLength = ceil(median(winBounds(preInds,2)-winBounds(preInds,1)));
    beats = zeros(3,hbLength,max(steps));
    for ii = 1:max(steps)
        beats(:,:,ii) = timeWarp(signal(winBounds(ii,1):winBounds(ii,2),[ecg_ind,abp_ind])',hbLength);
%         beats(:,:,ii) = bsxfun(@minus,beats(:,:,ii),mean(beats(:,:,ii),2));
    end
    beats = bsxfun(@minus,beats,mean(beats,2));
    prototype = squeeze(mean(beats(:,:,preInds),3));
    temp_preSCV = cat(1,repmat(prototype,1,1,length(preInds)),beats(:,:,preInds));
    temp_postSCV = cat(1,repmat(prototype,1,1,length(postInds)),beats(:,:,postInds));
    preSCV = zeros(6,6,length(preInds));
    for ii = 1:length(preInds)
        preSCV(:,:,ii) = cov(temp_preSCV(:,:,ii)');
    end
    postSCV = zeros(6,6,length(postInds));
    for ii = 1:length(postInds)
        postSCV(:,:,ii) = cov(temp_postSCV(:,:,ii)');
    end
    abpBased.preSCV = preSCV;
    abpBased.postSCV = postSCV;
    varargout{1} = abpBased;
    
    catch
        varargout{1} = [];
    end
else
    varargout{1} = [];
end

if ~isempty(ppg_ind)
    try
    test1 = find(isnan(signal(:,ecg_ind)));
    if ~isempty(test1)
        error('Bad ECG Lead');
    end
%     phase_ppg = angle(hilbert(signal(:,ecg_ind(1))-mean(signal(:,ecg_ind(1)))));
    phase_ppg = angle(hilbert(signal(:,ppg_ind)-mean(signal(:,ppg_ind))));
    steps = ceil(unwrap(phase_ppg,3*pi/2)./(2*pi));
    winBounds = zeros(max(steps),2);
    for ii = 1:max(steps)
        winBounds(ii,1) = find(steps == ii,1,'first');
        winBounds(ii,2) = find(steps == ii,1,'last');
    end
%     preInds = find(winBounds(:,2)<=N0_d);
    preInds = find(winBounds(:,2)<=N1);
    postInds = find(winBounds(:,1)>=N0_d & winBounds(:,1)<N_d);
    hbLength = ceil(median(winBounds(preInds,2)-winBounds(preInds,1)));
    beats = zeros(3,hbLength,max(steps));
    for ii = 1:max(steps)
        beats(:,:,ii) = timeWarp(signal(winBounds(ii,1):winBounds(ii,2),[ecg_ind,ppg_ind])',hbLength);
    end
    beats = bsxfun(@minus,beats,mean(beats,2));
    prototype = squeeze(mean(beats(:,:,preInds),3));
    temp_preSCV = cat(1,repmat(prototype,1,1,length(preInds)),beats(:,:,preInds));
    temp_postSCV = cat(1,repmat(prototype,1,1,length(postInds)),beats(:,:,postInds));
    preSCV = zeros(6,6,length(preInds));
    for ii = 1:length(preInds)
        preSCV(:,:,ii) = cov(temp_preSCV(:,:,ii)');
    end
    postSCV = zeros(6,6,length(postInds));
    for ii = 1:length(postInds)
        postSCV(:,:,ii) = cov(temp_postSCV(:,:,ii)');
    end
    ppgBased.preSCV = preSCV;
    ppgBased.postSCV = postSCV;
    varargout{2} = ppgBased;
    catch
        varargout{2} = [];
    end
else
    varargout{2} = [];
end


confidence = [];
warning('off','MATLAB:funm:nonPosRealEig')
warning('off','MATLAB:funm:zeroEig')
if exist('abpBased')
    if isfield(abpBased,'postSCV')
        if ~isempty(abpBased.postSCV)
            testVals = processCovMats(abpBased.postSCV);
            if ~isempty(testVals)
                [confs,~] = manClass(testVals,'abp',alarm_type);
                confidence = cat(2,confidence,confs);
            end
        end
    end
end
if exist('ppgBased')
    if isfield(ppgBased,'postSCV')
        if ~isempty(ppgBased.postSCV)
            testVals = processCovMats(ppgBased.postSCV);
            if ~isempty(testVals)
                [confs,~] = manClass(testVals,'ppg',alarm_type);
                confidence = cat(2,confidence,confs);
            end
        end
    end
end
if isempty(confidence)
    alarmResult = 1;
elseif mean(confidence >= 0)
    alarmResult = 1;
else
    alarmResult = 0;
end
warning('on','MATLAB:funm:nonPosRealEig')
warning('on','MATLAB:funm:zeroEig')

% Write result to answers.txt
fid = fopen(answers, 'a');
if (fid == -1)
    error('Could not open answer file');
end

% Get base name of record (without directories)
i = strfind(recordName, filesep);
if (~isempty(i))
    basename = recordName(i(end)+1 : end);
else
    basename = recordName;
end

fprintf(fid, '%s,%d\n', basename, alarmResult);
fclose(fid);



end

%%%%%%%%%%%% Helper Functions %%%%%%%%%%%%%%%%%%%%%
function ind=get_index(description,pattern)
M=length(description);
tmp_ind=strfind(description,pattern);
ind=[];
for m=1:M
    if(~isempty(tmp_ind{m}))
        ind(end+1)=m;
    end
end
end

function out = timeWarp(signal,newLength)
%     signal2 = [signal(:,1:(end-1));signal(:,2:end)];
    oldLength = size(signal,2);
    times = linspace(1,oldLength,newLength);
    out = zeros(size(signal,1),newLength);
    for ii = 1:length(times)
        out(:,ii) = (1-rem(times(ii),1))*signal(:,floor(times(ii))) + rem(times(ii),1)*signal(:,ceil(times(ii)));
    end
end

function [C,varargout] = multiConvolveFFT(filters,y)
% filters are an [k,n] bank of functions, where n is length of each filter
% and k is the number of filters. y is a [1,N] signal vector.
% returns [k,N] (absolute value) convolutions.
% optionaly returns the phase, or argument, of the complex convolutions.
    nn = size(filters,2) + length(y); % nn = n+N
    yF = fft(y,nn);
    fF = fft(filters,nn,2);
    C = ifft(bsxfun(@times,yF,fF),[],2);
    C = C(:,( floor(size(filters,2)/2+1) + (0:(length(y)-1)) ));
    varargout{1} = angle(C);
    %C = abs(C);
    
end

function out = processCovMats(in)
% in is an [n,n,N] array of N [n,n] positive definite symmetric matrices;
out = [];
for kk = 1:size(in,3)
    temp = nonzeros(triu(logm(squeeze(in(:,:,kk)))));
    if find(isnan(temp)==1)
       
    else
        out = cat(2,out,real(temp));
    end
end
end